@@ -17,7 +17,7 @@
 #include <QSizePolicy>
 #include <QSpinBox>
 #include <QStandardPaths>
-#include <QTextCodec>
+#include <QStringDecoder>
 #include <QVBoxLayout>
 
 GeneralConf::GeneralConf(QWidget* parent)
@@ -32,35 +32,46 @@ GeneralConf::GeneralConf(QWidget* parent)
     // It must be initialized before the checkboxes.
     initScrollArea();
 
-    initShowHelp();
-    initSaveLastRegion();
-    initShowSidePanelButton();
-    initShowDesktopNotification();
-    initShowTrayIcon();
-    initHistoryConfirmationToDelete();
-    initCheckForUpdates();
     initAutostart();
-    initShowStartupLaunchMessage();
-    initCopyAndCloseAfterUpload();
-    initCopyPathAfterSave();
-    initAntialiasingPinZoom();
-    initUploadWithoutConfirmation();
-    initUseJpgForClipboard();
-    initSaveAfterCopy();
-    initUploadHistoryMax();
-    initUndoLimit();
-    initUploadClientSecret();
-    initAllowMultipleGuiInstances();
 #if !defined(Q_OS_WIN)
     initAutoCloseIdleDaemon();
 #endif
-    initPredefinedColorPaletteLarge();
+    initShowTrayIcon();
+    initUseGrimAdapter();
+    initShowDesktopNotification();
+    initShowAbortNotification();
+#if !defined(DISABLE_UPDATE_CHECKER)
+    initCheckForUpdates();
+#endif
+    initShowStartupLaunchMessage();
+    initShowQuitPrompt();
+    initAllowMultipleGuiInstances();
+    initSaveLastRegion();
+    initShowHelp();
+    initShowSidePanelButton();
+    initUseJpgForClipboard();
     initCopyOnDoubleClick();
+    initSaveAfterCopy();
+    initCopyPathAfterSave();
+    initAntialiasingPinZoom();
+    initUndoLimit();
+    initInsecurePixelate();
+#ifdef ENABLE_IMGUR
+    initCopyAndCloseAfterUpload();
+    initUploadWithoutConfirmation();
+    initHistoryConfirmationToDelete();
+    initUploadHistoryMax();
+    initUploadClientSecret();
+#endif
+    initPredefinedColorPaletteLarge();
+    initShowSelectionGeometry();
 
     m_layout->addStretch();
 
     initShowMagnifier();
     initSquareMagnifier();
+    initJpegQuality();
+    initReverseArrow();
     // this has to be at the end
     initConfigButtons();
     updateComponents();
@@ -72,20 +83,29 @@ void GeneralConf::_updateComponents(bool allowEmptySavePath)
     m_helpMessage->setChecked(config.showHelp());
     m_sidePanelButton->setChecked(config.showSidePanelButton());
     m_sysNotifications->setChecked(config.showDesktopNotification());
+    m_abortNotifications->setChecked(config.showAbortNotification());
     m_autostart->setChecked(config.startupLaunch());
-    m_copyAndCloseAfterUpload->setChecked(config.copyAndCloseAfterUpload());
     m_saveAfterCopy->setChecked(config.saveAfterCopy());
     m_copyPathAfterSave->setChecked(config.copyPathAfterSave());
     m_antialiasingPinZoom->setChecked(config.antialiasingPinZoom());
     m_useJpgForClipboard->setChecked(config.useJpgForClipboard());
+    m_copyOnDoubleClick->setChecked(config.copyOnDoubleClick());
+#ifdef ENABLE_IMGUR
     m_uploadWithoutConfirmation->setChecked(config.uploadWithoutConfirmation());
+    m_copyURLAfterUpload->setChecked(config.copyURLAfterUpload());
     m_historyConfirmationToDelete->setChecked(
       config.historyConfirmationToDelete());
+
+    m_uploadHistoryMax->setValue(config.uploadHistoryMax());
+#endif
+#if !defined(DISABLE_UPDATE_CHECKER)
     m_checkForUpdates->setChecked(config.checkForUpdates());
+#endif
     m_allowMultipleGuiInstances->setChecked(config.allowMultipleGuiInstances());
     m_showMagnifier->setChecked(config.showMagnifier());
     m_squareMagnifier->setChecked(config.squareMagnifier());
     m_saveLastRegion->setChecked(config.saveLastRegion());
+    m_reverseArrow->setChecked(config.reverseArrow());
 
 #if !defined(Q_OS_WIN)
     m_autoCloseIdleDaemon->setChecked(config.autoCloseIdleDaemon());
@@ -94,8 +114,8 @@ void GeneralConf::_updateComponents(bool allowEmptySavePath)
     m_predefinedColorPaletteLarge->setChecked(
       config.predefinedColorPaletteLarge());
     m_showStartupLaunchMessage->setChecked(config.showStartupLaunchMessage());
+    m_showQuitPrompt->setChecked(config.showQuitPrompt());
     m_screenshotPathFixedCheck->setChecked(config.savePathFixed());
-    m_uploadHistoryMax->setValue(config.uploadHistoryMax());
     m_undoLimit->setValue(config.undoLimit());
 
     if (allowEmptySavePath || !config.savePath().isEmpty()) {
@@ -104,6 +124,10 @@ void GeneralConf::_updateComponents(bool allowEmptySavePath)
 #if defined(Q_OS_LINUX) || defined(Q_OS_UNIX)
     m_showTray->setChecked(!config.disabledTrayIcon());
 #endif
+
+#if defined(Q_OS_LINUX)
+    m_useGrimAdapter->setChecked(config.useGrimAdapter());
+#endif
 }
 
 void GeneralConf::updateComponents()
@@ -131,10 +155,22 @@ void GeneralConf::showDesktopNotificationChanged(bool checked)
     ConfigHandler().setShowDesktopNotification(checked);
 }
 
+void GeneralConf::showAbortNotificationChanged(bool checked)
+{
+    ConfigHandler().setShowAbortNotification(checked);
+}
+
+void GeneralConf::useGrimAdapter(bool checked)
+{
+    ConfigHandler().useGrimAdapter(checked);
+}
+
+#if !defined(DISABLE_UPDATE_CHECKER)
 void GeneralConf::checkForUpdatesChanged(bool checked)
 {
     ConfigHandler().setCheckForUpdates(checked);
 }
+#endif
 
 void GeneralConf::allowMultipleGuiInstancesChanged(bool checked)
 {
@@ -158,12 +194,12 @@ void GeneralConf::importConfiguration()
         return;
     }
     QFile file(fileName);
-    QTextCodec* codec = QTextCodec::codecForLocale();
     if (!file.open(QFile::ReadOnly)) {
         QMessageBox::about(this, tr("Error"), tr("Unable to read file."));
         return;
     }
-    QString text = codec->toUnicode(file.readAll());
+    QStringDecoder decoder(QStringDecoder::System);
+    QString text = decoder(file.readAll());
     file.close();
 
     QFile config(ConfigHandler().configFilePath());
@@ -171,7 +207,8 @@ void GeneralConf::importConfiguration()
         QMessageBox::about(this, tr("Error"), tr("Unable to write file."));
         return;
     }
-    config.write(codec->fromUnicode(text));
+    QStringEncoder encoder(QStringEncoder::System);
+    config.write(encoder(text));
     config.close();
 }
 
@@ -244,9 +281,10 @@ void GeneralConf::initShowHelp()
 
 void GeneralConf::initSaveLastRegion()
 {
-    m_saveLastRegion = new QCheckBox(tr("Use last region"), this);
-    m_saveLastRegion->setToolTip(tr("Uses the last region as the default "
-                                    "selection for the next screenshot"));
+    m_saveLastRegion = new QCheckBox(tr("Use last region for GUI mode"), this);
+    m_saveLastRegion->setToolTip(
+      tr("Use the last region as the default selection for the next screenshot "
+         "in GUI mode"));
     m_scrollAreaLayout->addWidget(m_saveLastRegion);
 
     connect(m_saveLastRegion,
@@ -280,6 +318,18 @@ void GeneralConf::initShowDesktopNotification()
             &GeneralConf::showDesktopNotificationChanged);
 }
 
+void GeneralConf::initShowAbortNotification()
+{
+    m_abortNotifications = new QCheckBox(tr("Show abort notifications"), this);
+    m_abortNotifications->setToolTip(tr("Enable abort notifications"));
+    m_scrollAreaLayout->addWidget(m_abortNotifications);
+
+    connect(m_abortNotifications,
+            &QCheckBox::clicked,
+            this,
+            &GeneralConf::showAbortNotificationChanged);
+}
+
 void GeneralConf::initShowTrayIcon()
 {
 #if defined(Q_OS_LINUX) || defined(Q_OS_UNIX)
@@ -293,6 +343,24 @@ void GeneralConf::initShowTrayIcon()
 #endif
 }
 
+void GeneralConf::initUseGrimAdapter()
+{
+#if defined(Q_OS_LINUX)
+    m_useGrimAdapter =
+      new QCheckBox(tr("Use grim to capture screenshots"), this);
+    m_useGrimAdapter->setToolTip(
+      tr("Grim is a wayland only utility to capture screens based on the "
+         "screencopy protocol. Generally only enable on minimal wayland window "
+         "managers like sway, hyprland, etc."));
+    m_scrollAreaLayout->addWidget(m_useGrimAdapter);
+
+    connect(m_useGrimAdapter,
+            &QCheckBox::clicked,
+            this,
+            &GeneralConf::useGrimAdapter);
+#endif
+}
+
 void GeneralConf::initHistoryConfirmationToDelete()
 {
     m_historyConfirmationToDelete = new QCheckBox(
@@ -338,6 +406,7 @@ void GeneralConf::initConfigButtons()
             &GeneralConf::resetConfiguration);
 }
 
+#if !defined(DISABLE_UPDATE_CHECKER)
 void GeneralConf::initCheckForUpdates()
 {
     m_checkForUpdates = new QCheckBox(tr("Automatic check for updates"), this);
@@ -349,6 +418,7 @@ void GeneralConf::initCheckForUpdates()
             this,
             &GeneralConf::checkForUpdatesChanged);
 }
+#endif
 
 void GeneralConf::initAllowMultipleGuiInstances()
 {
@@ -366,9 +436,9 @@ void GeneralConf::initAllowMultipleGuiInstances()
 void GeneralConf::initAutoCloseIdleDaemon()
 {
     m_autoCloseIdleDaemon = new QCheckBox(
-      tr("Automatically close daemon when it is not needed"), this);
-    m_autoCloseIdleDaemon->setToolTip(
-      tr("Automatically close daemon when it is not needed"));
+      tr("Automatically unload from memory when it is not needed"), this);
+    m_autoCloseIdleDaemon->setToolTip(tr(
+      "Automatically close daemon (background process) when it is not needed"));
     m_scrollAreaLayout->addWidget(m_autoCloseIdleDaemon);
     connect(m_autoCloseIdleDaemon,
             &QCheckBox::clicked,
@@ -378,9 +448,9 @@ void GeneralConf::initAutoCloseIdleDaemon()
 
 void GeneralConf::initAutostart()
 {
-    m_autostart = new QCheckBox(tr("Launch at startup"), this);
-    m_autostart->setToolTip(
-      tr("Launch Flameshot daemon when computer is booted"));
+    m_autostart = new QCheckBox(tr("Launch in background at startup"), this);
+    m_autostart->setToolTip(tr(
+      "Launch Flameshot daemon (background process) when computer is booted"));
     m_scrollAreaLayout->addWidget(m_autostart);
 
     connect(
@@ -402,6 +472,19 @@ void GeneralConf::initShowStartupLaunchMessage()
     });
 }
 
+void GeneralConf::initShowQuitPrompt()
+{
+    m_showQuitPrompt = new QCheckBox(tr("Ask before quit capture"), this);
+    ConfigHandler config;
+    m_showQuitPrompt->setToolTip(
+      tr("Show the confirmation prompt before ESC quit"));
+    m_scrollAreaLayout->addWidget(m_showQuitPrompt);
+
+    connect(m_showQuitPrompt, &QCheckBox::clicked, [](bool checked) {
+        ConfigHandler().setShowQuitPrompt(checked);
+    });
+}
+
 void GeneralConf::initPredefinedColorPaletteLarge()
 {
     m_predefinedColorPaletteLarge =
@@ -418,7 +501,8 @@ void GeneralConf::initPredefinedColorPaletteLarge()
 void GeneralConf::initCopyOnDoubleClick()
 {
     m_copyOnDoubleClick = new QCheckBox(tr("Copy on double click"), this);
-    m_copyOnDoubleClick->setToolTip(tr("Enable Copy on Double Click"));
+    m_copyOnDoubleClick->setToolTip(
+      tr("Enable Copy to clipboard on Double Click"));
     m_scrollAreaLayout->addWidget(m_copyOnDoubleClick);
 
     connect(m_copyOnDoubleClick, &QCheckBox::clicked, [](bool checked) {
@@ -428,14 +512,13 @@ void GeneralConf::initCopyOnDoubleClick()
 
 void GeneralConf::initCopyAndCloseAfterUpload()
 {
-    m_copyAndCloseAfterUpload =
-      new QCheckBox(tr("Copy URL after upload"), this);
-    m_copyAndCloseAfterUpload->setToolTip(
-      tr("Copy URL and close window after uploading was successful"));
-    m_scrollAreaLayout->addWidget(m_copyAndCloseAfterUpload);
+    m_copyURLAfterUpload = new QCheckBox(tr("Copy URL after upload"), this);
+    m_copyURLAfterUpload->setToolTip(
+      tr("Copy URL after uploading was successful"));
+    m_scrollAreaLayout->addWidget(m_copyURLAfterUpload);
 
-    connect(m_copyAndCloseAfterUpload, &QCheckBox::clicked, [](bool checked) {
-        ConfigHandler().setCopyAndCloseAfterUpload(checked);
+    connect(m_copyURLAfterUpload, &QCheckBox::clicked, [](bool checked) {
+        ConfigHandler().setCopyURLAfterUpload(checked);
     });
 }
 
@@ -476,9 +559,9 @@ void GeneralConf::initSaveAfterCopy()
     m_screenshotPathFixedCheck =
       new QCheckBox(tr("Use fixed path for screenshots to save"), this);
     connect(m_screenshotPathFixedCheck,
-            SIGNAL(toggled(bool)),
+            &QCheckBox::toggled,
             this,
-            SLOT(togglePathFixed()));
+            &GeneralConf::togglePathFixed);
 
     vboxLayout->addLayout(pathLayout);
     vboxLayout->addWidget(m_screenshotPathFixedCheck);
@@ -490,7 +573,7 @@ void GeneralConf::initSaveAfterCopy()
     m_setSaveAsFileExtension = new QComboBox(this);
 
     QStringList imageFormatList;
-    foreach (auto mimeType, QImageWriter::supportedImageFormats())
+    for (const auto& mimeType : QImageWriter::supportedImageFormats())
         imageFormatList.append(mimeType);
 
     m_setSaveAsFileExtension->addItems(imageFormatList);
@@ -500,9 +583,9 @@ void GeneralConf::initSaveAfterCopy()
     m_setSaveAsFileExtension->setCurrentIndex(currentIndex);
 
     connect(m_setSaveAsFileExtension,
-            SIGNAL(currentTextChanged(QString)),
+            &QComboBox::currentTextChanged,
             this,
-            SLOT(setSaveAsFileExtension(QString)));
+            &GeneralConf::setSaveAsFileExtension);
 
     extensionLayout->addWidget(m_setSaveAsFileExtension);
     vboxLayout->addLayout(extensionLayout);
@@ -529,9 +612,9 @@ void GeneralConf::initUploadHistoryMax()
       QStringLiteral("color: %1").arg(foreground));
 
     connect(m_uploadHistoryMax,
-            SIGNAL(valueChanged(int)),
+            static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
             this,
-            SLOT(uploadHistoryMaxChanged(int)));
+            &GeneralConf::uploadHistoryMaxChanged);
     vboxLayout->addWidget(m_uploadHistoryMax);
 }
 
@@ -550,9 +633,9 @@ void GeneralConf::initUploadClientSecret()
       QStringLiteral("color: %1").arg(foreground));
     m_uploadClientKey->setText(ConfigHandler().uploadClientSecret());
     connect(m_uploadClientKey,
-            SIGNAL(editingFinished()),
+            &QLineEdit::editingFinished,
             this,
-            SLOT(uploadClientKeyEdited()));
+            &GeneralConf::uploadClientKeyEdited);
     vboxLayout->addWidget(m_uploadClientKey);
 }
 
@@ -581,7 +664,10 @@ void GeneralConf::initUndoLimit()
     QString foreground = this->palette().windowText().color().name();
     m_undoLimit->setStyleSheet(QStringLiteral("color: %1").arg(foreground));
 
-    connect(m_undoLimit, SIGNAL(valueChanged(int)), this, SLOT(undoLimit(int)));
+    connect(m_undoLimit,
+            static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+            this,
+            &GeneralConf::undoLimit);
 
     vboxLayout->addWidget(m_undoLimit);
 }
@@ -596,13 +682,9 @@ void GeneralConf::initUseJpgForClipboard()
     m_useJpgForClipboard =
       new QCheckBox(tr("Use JPG format for clipboard (PNG default)"), this);
     m_useJpgForClipboard->setToolTip(
-      tr("Use JPG format for clipboard (PNG default)"));
+      tr("Use lossy JPG format for clipboard (lossless PNG default)"));
     m_scrollAreaLayout->addWidget(m_useJpgForClipboard);
 
-#if defined(Q_OS_MACOS)
-    // FIXME - temporary fix to disable option for MacOS
-    m_useJpgForClipboard->hide();
-#endif
     connect(m_useJpgForClipboard,
             &QCheckBox::clicked,
             this,
@@ -660,7 +742,7 @@ void GeneralConf::initUploadWithoutConfirmation()
     });
 }
 
-const QString GeneralConf::chooseFolder(const QString pathDefault)
+const QString GeneralConf::chooseFolder(const QString& pathDefault)
 {
     QString path;
     if (pathDefault.isEmpty()) {
@@ -675,13 +757,13 @@ const QString GeneralConf::chooseFolder(const QString pathDefault)
     if (path.isEmpty()) {
         return path;
     }
-    if (!path.isEmpty()) {
-        if (!QFileInfo(path).isWritable()) {
-            QMessageBox::about(
-              this, tr("Error"), tr("Unable to write to directory."));
-            return QString();
-        }
+
+    if (!QFileInfo(path).isWritable()) {
+        QMessageBox::about(
+          this, tr("Error"), tr("Unable to write to directory."));
+        return QString();
     }
+
     return path;
 }
 
@@ -707,12 +789,129 @@ void GeneralConf::initSquareMagnifier()
     });
 }
 
+void GeneralConf::initShowSelectionGeometry()
+{
+    auto* tobox = new QHBoxLayout();
+
+    int timeout =
+      ConfigHandler().value("showSelectionGeometryHideTime").toInt();
+    m_xywhTimeout = new QSpinBox();
+    m_xywhTimeout->setRange(0, INT_MAX);
+    m_xywhTimeout->setToolTip(
+      tr("Milliseconds before geometry display hides; 0 means do not hide"));
+    m_xywhTimeout->setValue(timeout);
+    tobox->addWidget(m_xywhTimeout);
+    tobox->addWidget(new QLabel(tr("Set geometry display timeout (ms)")));
+
+    m_scrollAreaLayout->addLayout(tobox);
+    connect(m_xywhTimeout,
+            static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+            this,
+            &GeneralConf::setSelGeoHideTime);
+
+    auto* box = new QGroupBox(tr("Selection Geometry Display"));
+    box->setFlat(true);
+    m_layout->addWidget(box);
+
+    auto* vboxLayout = new QVBoxLayout();
+    box->setLayout(vboxLayout);
+    auto* selGeoLayout = new QHBoxLayout();
+    selGeoLayout->addWidget(new QLabel(tr("Display Location")));
+    m_selectGeometryLocation = new QComboBox(this);
+
+    m_selectGeometryLocation->addItem(tr("None"), GeneralConf::xywh_none);
+    m_selectGeometryLocation->addItem(tr("Top Left"),
+                                      GeneralConf::xywh_top_left);
+    m_selectGeometryLocation->addItem(tr("Top Right"),
+                                      GeneralConf::xywh_top_right);
+    m_selectGeometryLocation->addItem(tr("Bottom Left"),
+                                      GeneralConf::xywh_bottom_left);
+    m_selectGeometryLocation->addItem(tr("Bottom Right"),
+                                      GeneralConf::xywh_bottom_right);
+    m_selectGeometryLocation->addItem(tr("Center"), GeneralConf::xywh_center);
+
+    // pick up int from config and use findData
+    int pos = ConfigHandler().value("showSelectionGeometry").toInt();
+    m_selectGeometryLocation->setCurrentIndex(
+      m_selectGeometryLocation->findData(pos));
+
+    connect(
+      m_selectGeometryLocation,
+      static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+      this,
+      &GeneralConf::setGeometryLocation);
+
+    selGeoLayout->addWidget(m_selectGeometryLocation);
+    vboxLayout->addLayout(selGeoLayout);
+    vboxLayout->addStretch();
+}
+
+void GeneralConf::initJpegQuality()
+{
+    auto* tobox = new QHBoxLayout();
+
+    int quality = ConfigHandler().value("jpegQuality").toInt();
+    m_jpegQuality = new QSpinBox();
+    m_jpegQuality->setRange(0, 100);
+    m_jpegQuality->setToolTip(tr("Quality range of 0-100; Higher number is "
+                                 "better quality and larger file size"));
+    m_jpegQuality->setValue(quality);
+    tobox->addWidget(m_jpegQuality);
+    tobox->addWidget(new QLabel(tr("JPEG Quality")));
+
+    m_scrollAreaLayout->addLayout(tobox);
+    connect(m_jpegQuality,
+            static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+            this,
+            &GeneralConf::setJpegQuality);
+}
+
+void GeneralConf::initReverseArrow()
+{
+    m_reverseArrow = new QCheckBox(tr("Reverse arrow"), this);
+    m_reverseArrow->setToolTip(tr("Draw the arrow head first"));
+    m_scrollAreaLayout->addWidget(m_reverseArrow);
+
+    connect(
+      m_reverseArrow, &QCheckBox::clicked, this, &GeneralConf::setReverseArrow);
+}
+
+void GeneralConf::initInsecurePixelate()
+{
+    m_insecurePixelate = new QCheckBox(tr("Insecure Pixelate"), this);
+    m_insecurePixelate->setToolTip(
+      tr("Draw the pixelation effect in an insecure but more asethetic way."));
+    m_insecurePixelate->setChecked(ConfigHandler().insecurePixelate());
+    m_scrollAreaLayout->addWidget(m_insecurePixelate);
+
+    connect(m_insecurePixelate,
+            &QCheckBox::clicked,
+            this,
+            &GeneralConf::setInsecurePixelate);
+}
+
+void GeneralConf::setSelGeoHideTime(int v)
+{
+    ConfigHandler().setValue("showSelectionGeometryHideTime", v);
+}
+
+void GeneralConf::setJpegQuality(int v)
+{
+    ConfigHandler().setJpegQuality(v);
+}
+
+void GeneralConf::setGeometryLocation(int index)
+{
+    ConfigHandler().setValue("showSelectionGeometry",
+                             m_selectGeometryLocation->itemData(index));
+}
+
 void GeneralConf::togglePathFixed()
 {
     ConfigHandler().setSavePathFixed(m_screenshotPathFixedCheck->isChecked());
 }
 
-void GeneralConf::setSaveAsFileExtension(QString extension)
+void GeneralConf::setSaveAsFileExtension(const QString& extension)
 {
     ConfigHandler().setSaveAsFileExtension(extension);
 }
@@ -721,3 +920,13 @@ void GeneralConf::useJpgForClipboardChanged(bool checked)
 {
     ConfigHandler().setUseJpgForClipboard(checked);
 }
+
+void GeneralConf::setReverseArrow(bool checked)
+{
+    ConfigHandler().setReverseArrow(checked);
+}
+
+void GeneralConf::setInsecurePixelate(bool checked)
+{
+    ConfigHandler().setInsecurePixelate(checked);
+}
\ No newline at end of file
