@@ -1,26 +1,17 @@
-// Copyright(c) 2017-2019 Alejandro Sirgo Rica & Contributors
-//
-// This file is part of Flameshot.
-//
-//     Flameshot is free software: you can redistribute it and/or modify
-//     it under the terms of the GNU General Public License as published by
-//     the Free Software Foundation, either version 3 of the License, or
-//     (at your option) any later version.
-//
-//     Flameshot is distributed in the hope that it will be useful,
-//     but WITHOUT ANY WARRANTY; without even the implied warranty of
-//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//     GNU General Public License for more details.
-//
-//     You should have received a copy of the GNU General Public License
-//     along with Flameshot.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
+// SPDX-FileCopyrightText: 2017-2019 Alejandro Sirgo Rica & Contributors
 
 #pragma once
 
-#include "src/utils/desktopfileparse.h"
 #include <QMap>
 #include <QWidget>
 
+#if defined(Q_OS_WIN)
+#include "src/utils/winlnkfileparse.h"
+#else
+#include "src/utils/desktopfileparse.h"
+#endif
+
 class QTabWidget;
 class QCheckBox;
 class QVBoxLayout;
@@ -44,8 +35,13 @@ private:
     void configureListView(QListWidget* widget);
     void addAppsToListWidget(QListWidget* widget,
                              const QVector<DesktopAppData>& appList);
+    void keyPressEvent(QKeyEvent* keyEvent) override;
 
+#if defined(Q_OS_WIN)
+    WinLnkFileParser m_parser;
+#else
     DesktopFileParser m_parser;
+#endif
     QPixmap m_pixmap;
     QString m_tempFile;
     bool m_keepOpen;
