@@ -77,9 +77,14 @@ static QMap<class QString, QSharedPointer<ValueHandler>>
     OPTION("showHelp"                    ,Bool               ( true          )),
     OPTION("showSidePanelButton"         ,Bool               ( true          )),
     OPTION("showDesktopNotification"     ,Bool               ( true          )),
+    OPTION("showAbortNotification"       ,Bool               ( true          )),
     OPTION("disabledTrayIcon"            ,Bool               ( false         )),
+    OPTION("useGrimAdapter"              ,Bool               ( false         )),
+    OPTION("disabledGrimWarning"         ,Bool               ( false         )),
     OPTION("historyConfirmationToDelete" ,Bool               ( true          )),
+#if !defined(DISABLE_UPDATE_CHECKER)
     OPTION("checkForUpdates"             ,Bool               ( true          )),
+#endif
     OPTION("allowMultipleGuiInstances"   ,Bool               ( false         )),
     OPTION("showMagnifier"               ,Bool               ( false         )),
     OPTION("squareMagnifier"             ,Bool               ( false         )),
@@ -88,30 +93,37 @@ static QMap<class QString, QSharedPointer<ValueHandler>>
 #endif
     OPTION("startupLaunch"               ,Bool               ( false         )),
     OPTION("showStartupLaunchMessage"    ,Bool               ( true          )),
-    OPTION("copyAndCloseAfterUpload"     ,Bool               ( true          )),
+    OPTION("showQuitPrompt"              ,Bool               ( false         )),
+    OPTION("copyURLAfterUpload"          ,Bool               ( true          )),
     OPTION("copyPathAfterSave"           ,Bool               ( false         )),
     OPTION("antialiasingPinZoom"         ,Bool               ( true          )),
     OPTION("useJpgForClipboard"          ,Bool               ( false         )),
     OPTION("uploadWithoutConfirmation"   ,Bool               ( false         )),
     OPTION("saveAfterCopy"               ,Bool               ( false         )),
-    OPTION("savePath"                    ,ExistingDir        (                   )),
+    OPTION("savePath"                    ,ExistingDir        (               )),
     OPTION("savePathFixed"               ,Bool               ( false         )),
-    OPTION("saveAsFileExtension"         ,SaveFileExtension  (                   )),
-    OPTION("saveLastRegion"              ,Bool               (false          )),
-    OPTION("uploadHistoryMax"            ,LowerBoundedInt    (0, 25               )),
-    OPTION("undoLimit"                   ,BoundedInt         (0, 999, 100    )),
-  // Interface tab
-    OPTION("uiColor"                     ,Color              ( {116, 0, 150}   )),
-    OPTION("contrastUiColor"             ,Color              ( {39, 0, 50}     )),
-    OPTION("contrastOpacity"             ,BoundedInt         ( 0, 255, 190    )),
+    OPTION("saveAsFileExtension"         ,SaveFileExtension  (               )),
+    OPTION("saveLastRegion"              ,Bool               ( false         )),
+    OPTION("uploadHistoryMax"            ,LowerBoundedInt    ( 0, 25         )),
+    OPTION("undoLimit"                   ,BoundedInt         ( 0, 999, 100   )),
+    // Interface tab
+    OPTION("uiLanguage"                  ,String             ( "auto"        )),
+    OPTION("uiColor"                     ,Color              ( {116, 0, 150} )),
+    OPTION("contrastUiColor"             ,Color              ( {39, 0, 50}   )),
+    OPTION("contrastOpacity"             ,BoundedInt         ( 0, 255, 190   )),
     OPTION("buttons"                     ,ButtonList         ( {}            )),
     // Filename Editor tab
     OPTION("filenamePattern"             ,FilenamePattern    ( {}            )),
     // Others
-    OPTION("drawThickness"               ,LowerBoundedInt    (1  , 3             )),
-    OPTION("drawFontSize"                ,LowerBoundedInt    (1  , 8             )),
+    // drawThickness shared by Pencil, Line, Arrow, Rectangular Selection, Circle
+    OPTION("drawThickness"               ,LowerBoundedInt    ( 1, 3          )),
+    OPTION("drawFontSize"                ,LowerBoundedInt    ( 1, 8          )),
+    OPTION("drawCircleCounterSize"       ,LowerBoundedInt    ( 1, 1          )),
+    OPTION("drawPixelateSize"            ,LowerBoundedInt    ( 1, 2          )),
+    OPTION("drawRectangleSize"           ,LowerBoundedInt    ( 1, 1          )),
+    OPTION("drawMarkerSize"              ,LowerBoundedInt    ( 1, 5          )),
     OPTION("drawColor"                   ,Color              ( Qt::red       )),
-    OPTION("userColors"                  ,UserColors(3,        17            )),
+    OPTION("userColors"                  ,UserColors         ( 3, 17         )),
     OPTION("ignoreUpdateToVersion"       ,String             ( ""            )),
     OPTION("keepOpenAppLauncher"         ,Bool               ( false         )),
     OPTION("fontFamily"                  ,String             ( ""            )),
@@ -120,7 +132,12 @@ static QMap<class QString, QSharedPointer<ValueHandler>>
     // NOTE: If another tool size is added besides drawThickness and
     // drawFontSize, remember to update ConfigHandler::toolSize
     OPTION("copyOnDoubleClick"           ,Bool               ( false         )),
-    OPTION("uploadClientSecret"          ,String             ( "313baf0c7b4d3ff"            )),
+    OPTION("uploadClientSecret"          ,String             ( "313baf0c7b4d3ff" )),
+    OPTION("showSelectionGeometry"       , BoundedInt        ( 0, 5, 4       )),
+    OPTION("showSelectionGeometryHideTime", LowerBoundedInt  ( 0, 3000       )),
+    OPTION("jpegQuality"                 , BoundedInt        ( 0,100,75      )),
+    OPTION("reverseArrow"                ,Bool               ( false         )),
+    OPTION("insecurePixelate"            ,Bool               ( false         )),
 };
 
 static QMap<QString, QSharedPointer<KeySequence>> recognizedShortcuts = {
@@ -138,7 +155,10 @@ static QMap<QString, QSharedPointer<KeySequence>> recognizedShortcuts = {
     SHORTCUT("TYPE_SAVE"                ,   "Ctrl+S"                ),
     SHORTCUT("TYPE_ACCEPT"              ,   "Return"                ),
     SHORTCUT("TYPE_EXIT"                ,   "Ctrl+Q"                ),
+    SHORTCUT("TYPE_CANCEL"              ,   "Ctrl+Backspace"        ),
+#ifdef ENABLE_IMGUR
     SHORTCUT("TYPE_IMAGEUPLOADER"       ,                           ),
+#endif
 #if !defined(Q_OS_MACOS)
     SHORTCUT("TYPE_OPEN_APP"            ,   "Ctrl+O"                ),
 #endif
@@ -147,10 +167,15 @@ static QMap<QString, QSharedPointer<KeySequence>> recognizedShortcuts = {
     SHORTCUT("TYPE_REDO"                ,   "Ctrl+Shift+Z"          ),
     SHORTCUT("TYPE_TEXT"                ,   "T"                     ),
     SHORTCUT("TYPE_TOGGLE_PANEL"        ,   "Space"                 ),
+    SHORTCUT("TYPE_GRAB_COLOR"          ,   "G"                     ),
     SHORTCUT("TYPE_RESIZE_LEFT"         ,   "Shift+Left"            ),
     SHORTCUT("TYPE_RESIZE_RIGHT"        ,   "Shift+Right"           ),
     SHORTCUT("TYPE_RESIZE_UP"           ,   "Shift+Up"              ),
     SHORTCUT("TYPE_RESIZE_DOWN"         ,   "Shift+Down"            ),
+    SHORTCUT("TYPE_SYM_RESIZE_LEFT"     ,   "Ctrl+Shift+Left"       ),
+    SHORTCUT("TYPE_SYM_RESIZE_RIGHT"    ,   "Ctrl+Shift+Right"      ),
+    SHORTCUT("TYPE_SYM_RESIZE_UP"       ,   "Ctrl+Shift+Up"         ),
+    SHORTCUT("TYPE_SYM_RESIZE_DOWN"     ,   "Ctrl+Shift+Down"       ),
     SHORTCUT("TYPE_SELECT_ALL"          ,   "Ctrl+A"                ),
     SHORTCUT("TYPE_MOVE_LEFT"           ,   "Left"                  ),
     SHORTCUT("TYPE_MOVE_RIGHT"          ,   "Right"                 ),
@@ -165,7 +190,6 @@ static QMap<QString, QSharedPointer<KeySequence>> recognizedShortcuts = {
     SHORTCUT("TYPE_DELETE_CURRENT_TOOL" ,   "Delete"                ),
 #endif
     SHORTCUT("TYPE_PIN"                 ,                           ),
-    SHORTCUT("TYPE_SELECTIONINDICATOR"  ,                           ),
     SHORTCUT("TYPE_SIZEINCREASE"        ,                           ),
     SHORTCUT("TYPE_SIZEDECREASE"        ,                           ),
     SHORTCUT("TYPE_CIRCLECOUNT"         ,                           ),
@@ -313,16 +337,25 @@ void ConfigHandler::setStartupLaunch(const bool start)
 
 void ConfigHandler::setAllTheButtons()
 {
-    QList<CaptureTool::Type> buttons =
+    QList<CaptureTool::Type> buttonlist =
       CaptureToolButton::getIterableButtonTypes();
-    setValue(QStringLiteral("buttons"), QVariant::fromValue(buttons));
+    setValue(QStringLiteral("buttons"), QVariant::fromValue(buttonlist));
 }
 
 void ConfigHandler::setToolSize(CaptureTool::Type toolType, int size)
 {
     if (toolType == CaptureTool::TYPE_TEXT) {
         setDrawFontSize(size);
+    } else if (toolType == CaptureTool::TYPE_RECTANGLE) {
+        setDrawRectangleSize(size);
+    } else if (toolType == CaptureTool::TYPE_MARKER) {
+        setDrawMarkerSize(size);
+    } else if (toolType == CaptureTool::TYPE_PIXELATE) {
+        setDrawPixelateSize(size);
+    } else if (toolType == CaptureTool::TYPE_CIRCLECOUNT) {
+        setDrawCircleCounterSize(size);
     } else if (toolType != CaptureTool::NONE) {
+        // All other tools are sharing the same size
         setDrawThickness(size);
     }
 }
@@ -331,7 +364,16 @@ int ConfigHandler::toolSize(CaptureTool::Type toolType)
 {
     if (toolType == CaptureTool::TYPE_TEXT) {
         return drawFontSize();
+    } else if (toolType == CaptureTool::TYPE_RECTANGLE) {
+        return drawRectangleSize();
+    } else if (toolType == CaptureTool::TYPE_MARKER) {
+        return drawMarkerSize();
+    } else if (toolType == CaptureTool::TYPE_PIXELATE) {
+        return drawPixelateSize();
+    } else if (toolType == CaptureTool::TYPE_CIRCLECOUNT) {
+        return drawCircleCounterSize();
     } else {
+        // All other tools are sharing the same size
         return drawThickness();
     }
 }
@@ -345,7 +387,7 @@ QString ConfigHandler::filenamePatternDefault()
 
 void ConfigHandler::setDefaultSettings()
 {
-    foreach (const QString& key, m_settings.allKeys()) {
+    for (const auto& key : m_settings.allKeys()) {
         if (isShortcut(key)) {
             // Do not reset Shortcuts
             continue;
@@ -368,7 +410,7 @@ bool ConfigHandler::setShortcut(const QString& actionName,
     qDebug() << actionName;
     static QVector<QKeySequence> reservedShortcuts = {
 #if defined(Q_OS_MACOS)
-        Qt::CTRL + Qt::Key_Backspace,
+        Qt::CTRL | Qt::Key_Backspace,
         Qt::Key_Escape,
 #else
         Qt::Key_Backspace,
@@ -380,16 +422,16 @@ bool ConfigHandler::setShortcut(const QString& actionName,
         return false;
     }
 
-    bool error = false;
+    bool errorFlag = false;
 
     m_settings.beginGroup(CONFIG_GROUP_SHORTCUTS);
     if (shortcut.isEmpty()) {
         setValue(actionName, "");
     } else if (reservedShortcuts.contains(QKeySequence(shortcut))) {
         // do not allow to set reserved shortcuts
-        error = true;
+        errorFlag = true;
     } else {
-        error = false;
+        errorFlag = false;
         // Make no difference for Return and Enter keys
         QString newShortcut = KeySequence().value(shortcut).toString();
         for (auto& otherAction : m_settings.allKeys()) {
@@ -399,7 +441,7 @@ bool ConfigHandler::setShortcut(const QString& actionName,
             QString existingShortcut =
               KeySequence().value(m_settings.value(otherAction)).toString();
             if (newShortcut == existingShortcut) {
-                error = true;
+                errorFlag = true;
                 goto done;
             }
         }
@@ -407,7 +449,7 @@ bool ConfigHandler::setShortcut(const QString& actionName,
     }
 done:
     m_settings.endGroup();
-    return !error;
+    return !errorFlag;
 }
 
 QString ConfigHandler::shortcut(const QString& actionName)
@@ -472,25 +514,15 @@ void ConfigHandler::resetValue(const QString& key)
 
 QSet<QString>& ConfigHandler::recognizedGeneralOptions()
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
     auto keys = ::recognizedGeneralOptions.keys();
     static QSet<QString> options = QSet<QString>(keys.begin(), keys.end());
-#else
-    static QSet<QString> options =
-      QSet<QString>::fromList(::recognizedGeneralOptions.keys());
-#endif
     return options;
 }
 
 QSet<QString>& ConfigHandler::recognizedShortcutNames()
 {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
     auto keys = recognizedShortcuts.keys();
     static QSet<QString> names = QSet<QString>(keys.begin(), keys.end());
-#else
-    static QSet<QString> names =
-      QSet<QString>::fromList(recognizedShortcuts.keys());
-#endif
     return names;
 }
 
@@ -598,9 +630,7 @@ bool ConfigHandler::checkShortcutConflicts(AbstractLogger* log) const
                     reportedInLog.append(*key2);
                     *log << tr("Shortcut conflict: '%1' and '%2' "
                                "have the same shortcut: %3\n")
-                              .arg(*key1)
-                              .arg(*key2)
-                              .arg(value1);
+                              .arg(*key1, *key2, value1);
                 }
             }
         }
@@ -637,8 +667,7 @@ bool ConfigHandler::checkSemantics(AbstractLogger* log,
             }
             if (log != nullptr) {
                 *log << tr("Bad value in '%1'. Expected: %2\n")
-                          .arg(key)
-                          .arg(valueHandler->expected());
+                          .arg(key, valueHandler->expected());
             }
             if (offenders != nullptr) {
                 offenders->append(key);
@@ -779,7 +808,7 @@ bool ConfigHandler::isShortcut(const QString& key) const
            key.startsWith(QStringLiteral(CONFIG_GROUP_SHORTCUTS "/"));
 }
 
-QString ConfigHandler::baseName(QString key) const
+QString ConfigHandler::baseName(const QString& key) const
 {
     return QFileInfo(key).baseName();
 }
